@@ -2,7 +2,7 @@
 #define _SHAPEFILE_H_INCLUDED
 
 /******************************************************************************
- * $Id: shapefil.h,v 1.11 2003/05/21 04:03:54 warmerda Exp $
+ * $Id: shapefil.h,v 1.1.1.1 2005/09/01 18:21:44 rsbivand Exp $
  *
  * Project:  Shapelib
  * Purpose:  Primary include file for Shapelib.
@@ -37,12 +37,21 @@
  ******************************************************************************
  *
  * $Log: shapefil.h,v $
- * Revision 1.11  2003/05/21 04:03:54  warmerda
- * expand tabs
+ * Revision 1.1.1.1  2005/09/01 18:21:44  rsbivand
+ * Initial import.
  *
- * Revision 1.10  2003/04/21 19:02:56  warmerda
+ * Revision 1.14  2005/03/14 15:21:29  fwarmerdam
  * updated
  *
+ * Revision 1.30  2005/01/03 22:30:13  fwarmerdam
+ * added support for saved quadtrees
+ *
+ * Revision 1.29  2004/09/26 20:09:35  fwarmerdam
+ * avoid rcsid warnings
+ *
+ * Revision 1.28  2003/12/29 06:02:18  fwarmerdam
+ * added cpl_error.h option
+ *
  * Revision 1.27  2003/04/21 18:30:37  warmerda
  * added header write/update public methods
  *
@@ -129,6 +138,10 @@
 #include <dbmalloc.h>
 #endif
 
+#ifdef USE_CPL
+#include "cpl_error.h"
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -192,27 +205,38 @@ extern "C" {
 #  define SHPAPI_CALL1(x)      x SHPAPI_CALL
 #endif
     
+/* -------------------------------------------------------------------- */
+/*      Macros for controlling CVSID and ensuring they don't appear     */
+/*      as unreferenced variables resulting in lots of warnings.        */
+/* -------------------------------------------------------------------- */
+#ifndef DISABLE_CVSID
+#  define SHP_CVSID(string)     static char cpl_cvsid[] = string; \
+static char *cvsid_aw() { return( cvsid_aw() ? ((char *) NULL) : cpl_cvsid ); }
+#else
+#  define SHP_CVSID(string)
+#endif
+
 /************************************************************************/
 /*                             SHP Support.                             */
 /************************************************************************/
-typedef struct
+typedef	struct
 {
     FILE        *fpSHP;
-    FILE        *fpSHX;
+    FILE	*fpSHX;
 
-    int         nShapeType;                             /* SHPT_* */
+    int		nShapeType;				/* SHPT_* */
     
-    int         nFileSize;                              /* SHP file */
+    int		nFileSize;				/* SHP file */
 
     int         nRecords;
-    int         nMaxRecords;
-    int         *panRecOffset;
-    int         *panRecSize;
+    int		nMaxRecords;
+    int		*panRecOffset;
+    int		*panRecSize;
 
-    double      adBoundsMin[4];
-    double      adBoundsMax[4];
+    double	adBoundsMin[4];
+    double	adBoundsMax[4];
 
-    int         bUpdated;
+    int		bUpdated;
 
     unsigned char *pabyRec;
     int         nBufSize;
@@ -223,18 +247,18 @@ typedef SHPInfo * SHPHandle;
 /* -------------------------------------------------------------------- */
 /*      Shape types (nSHPType)                                          */
 /* -------------------------------------------------------------------- */
-#define SHPT_NULL       0
-#define SHPT_POINT      1
-#define SHPT_ARC        3
-#define SHPT_POLYGON    5
-#define SHPT_MULTIPOINT 8
-#define SHPT_POINTZ     11
-#define SHPT_ARCZ       13
-#define SHPT_POLYGONZ   15
+#define SHPT_NULL	0
+#define SHPT_POINT	1
+#define SHPT_ARC	3
+#define SHPT_POLYGON	5
+#define SHPT_MULTIPOINT	8
+#define SHPT_POINTZ	11
+#define SHPT_ARCZ	13
+#define SHPT_POLYGONZ	15
 #define SHPT_MULTIPOINTZ 18
-#define SHPT_POINTM     21
-#define SHPT_ARCM       23
-#define SHPT_POLYGONM   25
+#define SHPT_POINTM	21
+#define SHPT_ARCM	23
+#define SHPT_POLYGONM	25
 #define SHPT_MULTIPOINTM 28
 #define SHPT_MULTIPATCH 31
 
@@ -244,12 +268,12 @@ typedef SHPInfo * SHPHandle;
 /*      SHPP_RING.                                                      */
 /* -------------------------------------------------------------------- */
 
-#define SHPP_TRISTRIP   0
-#define SHPP_TRIFAN     1
-#define SHPP_OUTERRING  2
-#define SHPP_INNERRING  3
-#define SHPP_FIRSTRING  4
-#define SHPP_RING       5
+#define SHPP_TRISTRIP	0
+#define SHPP_TRIFAN	1
+#define SHPP_OUTERRING	2
+#define SHPP_INNERRING	3
+#define SHPP_FIRSTRING	4
+#define SHPP_RING	5
 
 /* -------------------------------------------------------------------- */
 /*      SHPObject - represents on shape (without attributes) read       */
@@ -257,29 +281,29 @@ typedef SHPInfo * SHPHandle;
 /* -------------------------------------------------------------------- */
 typedef struct
 {
-    int         nSHPType;
+    int		nSHPType;
 
-    int         nShapeId; /* -1 is unknown/unassigned */
+    int		nShapeId; /* -1 is unknown/unassigned */
 
-    int         nParts;
-    int         *panPartStart;
-    int         *panPartType;
+    int		nParts;
+    int		*panPartStart;
+    int		*panPartType;
     
-    int         nVertices;
-    double      *padfX;
-    double      *padfY;
-    double      *padfZ;
-    double      *padfM;
+    int		nVertices;
+    double	*padfX;
+    double	*padfY;
+    double	*padfZ;
+    double	*padfM;
 
-    double      dfXMin;
-    double      dfYMin;
-    double      dfZMin;
-    double      dfMMin;
+    double	dfXMin;
+    double	dfYMin;
+    double	dfZMin;
+    double	dfMMin;
 
-    double      dfXMax;
-    double      dfYMax;
-    double      dfZMax;
-    double      dfMMax;
+    double	dfXMax;
+    double	dfYMax;
+    double	dfZMax;
+    double	dfMMax;
 } SHPObject;
 
 /* -------------------------------------------------------------------- */
@@ -303,13 +327,16 @@ void SHPAPI_CALL
 void SHPAPI_CALL
       SHPComputeExtents( SHPObject * psObject );
 SHPObject SHPAPI_CALL1(*)
-      SHPCreateObject( int nSHPType, int nShapeId,
-                       int nParts, int * panPartStart, int * panPartType,
-                       int nVertices, double * padfX, double * padfY,
-                       double * padfZ, double * padfM );
+      SHPCreateObject( int nSHPType, int nShapeId, int nParts, 
+                       const int * panPartStart, const int * panPartType,
+                       int nVertices, 
+                       const double * padfX, const double * padfY,
+                       const double * padfZ, const double * padfM );
 SHPObject SHPAPI_CALL1(*)
       SHPCreateSimpleObject( int nSHPType, int nVertices,
-                             double * padfX, double * padfY, double * padfZ );
+                             const double * padfX, 
+                             const double * padfY, 
+                             const double * padfZ );
 
 int SHPAPI_CALL
       SHPRewindObject( SHPHandle hSHP, SHPObject * psObject );
@@ -327,21 +354,21 @@ const char SHPAPI_CALL1(*)
 /* -------------------------------------------------------------------- */
 
 /* this can be two or four for binary or quad tree */
-#define MAX_SUBNODE     4
+#define MAX_SUBNODE	4
 
 typedef struct shape_tree_node
 {
     /* region covered by this node */
-    double      adfBoundsMin[4];
-    double      adfBoundsMax[4];
+    double	adfBoundsMin[4];
+    double	adfBoundsMax[4];
 
     /* list of shapes stored at this node.  The papsShapeObj pointers
        or the whole list can be NULL */
-    int         nShapeCount;
-    int         *panShapeIds;
+    int		nShapeCount;
+    int		*panShapeIds;
     SHPObject   **papsShapeObj;
 
-    int         nSubNodes;
+    int		nSubNodes;
     struct shape_tree_node *apsSubNode[MAX_SUBNODE];
     
 } SHPTreeNode;
@@ -350,10 +377,11 @@ typedef struct
 {
     SHPHandle   hSHP;
     
-    int         nMaxDepth;
-    int         nDimension;
+    int		nMaxDepth;
+    int		nDimension;
+    int         nTotalCount;
     
-    SHPTreeNode *psRoot;
+    SHPTreeNode	*psRoot;
 } SHPTree;
 
 SHPTree SHPAPI_CALL1(*)
@@ -362,19 +390,19 @@ SHPTree SHPAPI_CALL1(*)
 void    SHPAPI_CALL
       SHPDestroyTree( SHPTree * hTree );
 
-int     SHPAPI_CALL
+int	SHPAPI_CALL
       SHPWriteTree( SHPTree *hTree, const char * pszFilename );
 SHPTree SHPAPI_CALL
       SHPReadTree( const char * pszFilename );
 
-int     SHPAPI_CALL
+int	SHPAPI_CALL
       SHPTreeAddObject( SHPTree * hTree, SHPObject * psObject );
-int     SHPAPI_CALL
+int	SHPAPI_CALL
       SHPTreeAddShapeId( SHPTree * hTree, SHPObject * psObject );
-int     SHPAPI_CALL
+int	SHPAPI_CALL
       SHPTreeRemoveShapeId( SHPTree * hTree, int nShapeId );
 
-void    SHPAPI_CALL
+void 	SHPAPI_CALL
       SHPTreeTrimExtraNodes( SHPTree * hTree );
 
 int    SHPAPI_CALL1(*)
@@ -385,31 +413,36 @@ int    SHPAPI_CALL1(*)
 int     SHPAPI_CALL
       SHPCheckBoundsOverlap( double *, double *, double *, double *, int );
 
+int SHPAPI_CALL1(*) 
+SHPSearchDiskTree( FILE *fp, 
+                   double *padfBoundsMin, double *padfBoundsMax,
+                   int *pnShapeCount );
+
 /************************************************************************/
 /*                             DBF Support.                             */
 /************************************************************************/
-typedef struct
+typedef	struct
 {
-    FILE        *fp;
+    FILE	*fp;
 
     int         nRecords;
 
-    int         nRecordLength;
-    int         nHeaderLength;
-    int         nFields;
-    int         *panFieldOffset;
-    int         *panFieldSize;
-    int         *panFieldDecimals;
-    char        *pachFieldType;
+    int		nRecordLength;
+    int		nHeaderLength;
+    int		nFields;
+    int		*panFieldOffset;
+    int		*panFieldSize;
+    int		*panFieldDecimals;
+    char	*pachFieldType;
 
-    char        *pszHeader;
+    char	*pszHeader;
 
-    int         nCurrentRecord;
-    int         bCurrentRecordModified;
-    char        *pszCurrentRecord;
+    int		nCurrentRecord;
+    int		bCurrentRecordModified;
+    char	*pszCurrentRecord;
     
-    int         bNoHeader;
-    int         bUpdated;
+    int		bNoHeader;
+    int		bUpdated;
 } DBFInfo;
 
 typedef DBFInfo * DBFHandle;
@@ -429,11 +462,11 @@ DBFHandle SHPAPI_CALL
 DBFHandle SHPAPI_CALL
       DBFCreate( const char * pszDBFFile );
 
-int     SHPAPI_CALL
+int	SHPAPI_CALL
       DBFGetFieldCount( DBFHandle psDBF );
-int     SHPAPI_CALL
+int	SHPAPI_CALL
       DBFGetRecordCount( DBFHandle psDBF );
-int     SHPAPI_CALL
+int	SHPAPI_CALL
       DBFAddField( DBFHandle hDBF, const char * pszFieldName,
                    DBFFieldType eType, int nWidth, int nDecimals );
 
@@ -444,9 +477,9 @@ DBFFieldType SHPAPI_CALL
 int SHPAPI_CALL
       DBFGetFieldIndex(DBFHandle psDBF, const char *pszFieldName);
 
-int     SHPAPI_CALL
+int 	SHPAPI_CALL
       DBFReadIntegerAttribute( DBFHandle hDBF, int iShape, int iField );
-double  SHPAPI_CALL
+double 	SHPAPI_CALL
       DBFReadDoubleAttribute( DBFHandle hDBF, int iShape, int iField );
 const char SHPAPI_CALL1(*)
       DBFReadStringAttribute( DBFHandle hDBF, int iShape, int iField );
@@ -469,7 +502,7 @@ int SHPAPI_CALL
 
 int SHPAPI_CALL
      DBFWriteLogicalAttribute( DBFHandle hDBF, int iShape, int iField,
-                               const char lFieldValue);
+			       const char lFieldValue);
 int SHPAPI_CALL
      DBFWriteAttributeDirectly(DBFHandle psDBF, int hEntity, int iField,
                                void * pValue );
@@ -481,7 +514,7 @@ int SHPAPI_CALL
 DBFHandle SHPAPI_CALL
       DBFCloneEmpty(DBFHandle psDBF, const char * pszFilename );
  
-void    SHPAPI_CALL
+void	SHPAPI_CALL
       DBFClose( DBFHandle hDBF );
 void    SHPAPI_CALL
       DBFUpdateHeader( DBFHandle hDBF );
